<?php
/**
 * @file
 * Contains \Drupal\publish_away\Form\PublishAwaySettingsForm.
 */
 
namespace Drupal\publish_away\Form;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\publish_away\Controller\Facebook;
// use Drupal\Component\Utility\NestedArray;
// use Drupal\Core\Datetime\DrupalDateTime;
// use Drupal\Core\Language\Language;
// use Drupal\Component\Utility\String;
 
class PublishAwaySettingsForm extends ConfigFormBase {
  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'system_publish_away_settings';
  }
  
  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, array &$form_state) {
    $publish_away_config = $this->configFactory->get('publish_away.settings');
    $form['#tree'];
    // Facebook connect.
    $account_set = $this->facebook_login();
    // Twitter connect.
    $account_set .= $this->twitter_login();
    // End Facebook connect.
    $form['fb_connect_fieldset'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Accounts'),
      '#collapsible' => TRUE,
    );
    $form['fb_connect_fieldset']['fb_connect'] = array(
      '#markup' => $account_set,
    );
    
    // Facebook settings.
    $form['facebook'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Facebook settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['facebook']['appId'] = array(
      '#title' => $this->t('AppId'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('facebook.appId'),
      '#required' => FALSE,
      '#disabled' => $publish_away_config->get('facebook.appId') != '',
    );
    $form['facebook']['secret'] = array(
      '#title' => $this->t('Secret key'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('facebook.secret'),
      '#required' => FALSE,
      '#disabled' => $publish_away_config->get('facebook.secret') != '',
    );
    // Twitter settings.
    $form['twitter'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Twitter settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,  
    );
    $form['twitter']['consumer_key'] = array(
      '#title' => $this->t('Consumer key'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('twitter.consumer_key'),
      '#required' => FALSE,
    );
    $form['twitter']['consumer_secret'] = array(
      '#title' => $this->t('Consumer secret key'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('twitter.consumer_secret'),
      '#required' => FALSE,
    );
    $form['twitter']['oauth_token'] = array(
      '#title' => $this->t('Access token'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('twitter.oauth_token'),
      '#required' => FALSE,
    );
    $form['twitter']['oauth_token_secret'] = array(
      '#title' => $this->t('Access token secret'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('twitter.oauth_token_secret'),
      '#required' => FALSE,
    );
    // Bitly settings.
    $form['bitly'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Bitly settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
      '#description' => $this->t('To shorten the links visit <a href="@bitlyurl" target="_blank" />@bitlyurl</a> and get your Bilty API Key', array('@bitlyurl' => 'https://bitly.com/a/your_api_key')),
    );
    $form['bitly']['username'] = array(
      '#title' => $this->t('Bitly username'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('bitly.username'),
      '#required' => FALSE,
    );
    $form['bitly']['apikey'] = array(
      '#title' => $this->t('Bitly API Key'),
      '#type' => 'textfield',
      '#default_value' => $publish_away_config->get('bitly.apikey'),
      '#required' => FALSE,
    );
    // Content types.
    $form['content'] = array(
      '#type' => 'fieldset',
      '#title' => $this->t('Content types settings'),
      '#collapsible' => TRUE,
    );
    $types = array();
    foreach (node_type_get_types() as $type => $info) {
      $types += array($type => $info->name);
    }
    $elements = drupal_map_assoc($types);
    $default_value_types = explode(',', $publish_away_config->get('content.types'));
    
    $form['content']['types'] = array(
      '#type' => 'checkboxes',
      '#title' => $this->t('Allow publish away action for the following content types :'),
      '#default_value' => $default_value_types,
      '#options' => $elements,
    );

    $form['#submit'][] = 'publish_away_admin_form_submit';

    return parent::buildForm($form, $form_state);
  }
  
  /**
   * Implements \Drupal\Core\Form\FormInterface::validateForm().
   */
  public function validateForm(array &$form, array &$form_state) {
    /*foreach ($form_state['values']['bundles'] as $bundle) {
      if (!in_array($bundle, array_keys(node_type_get_types()))) {
        form_set_error('bundles', $form_state, t('You have chosen an invalid node bundle, please check your selection and try again.'));
      }
      $keys = $form_state['values']['processed_keys'];
      if ($keys == '') {
        $this->setFormError('keys', $form_state, $this->t('Please enter some keywords.'));
        // Fall through to the form redirect.
      }
     
    }*/
  }
  
  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, array &$form_state) {
    $this->configFactory->get('publish_away.settings')
      ->set('facebook.appId', $form_state['values']['facebook']['appId'])
      ->set('facebook.secret', $form_state['values']['facebook']['secret'])
      ->set('twitter.consumer_key', $form_state['values']['twitter']['consumer_key'])
      ->set('twitter.consumer_secret', $form_state['values']['twitter']['consumer_key'])
      ->set('twitter.oauth_token', $form_state['values']['twitter']['oauth_token'])
      ->set('twitter.oauth_token_secret', $form_state['values']['twitter']['oauth_token_secret'])
      ->set('bitly.username', $form_state['values']['bitly']['username'])
      ->set('bitly.apikey', $form_state['values']['bitly']['apikey'])
      ->set('content.types', $form_state['values']['content']['types'])
      ->save();

    parent::submitForm($form, $form_state);
  }
  
  private function facebook_login() {
    $publish_away_config = $this->configFactory->get('publish_away.settings');
    // Facebook connect.
    $facebook_html = '';
    $token_fb = $publish_away_config->get('facebook.token');
    if (trim($token_fb) == '') {
      $config = array(
        'appId'  => $publish_away_config->get('facebook.appId'),
        'secret' => $publish_away_config->get('facebook.secret'),   
      );
      $facebook = new Facebook($config);
      if (!empty($facebook)) {
        $user_id = $facebook->getUser();
        if ($user_id) {
          // We have a user ID, so probably a logged in user.
          // If not, we'll get an exception, which we handle below.
          try {
            $user_profile = $facebook->api('/me','GET');
            $token = $facebook->getAccessToken();
            
            $this->configFactory->get('publish_away.settings')
              ->set('facebook.user_profile', $user_profile)
              ->set('facebook.id', $user_profile['id'])
              ->set('facebook.name', $user_profile['name'])
              ->set('facebook.token', $token)
              ->save();
          }
          catch(FacebookApiException $e) {
            // If the user is logged out, you can have a 
            // user ID even though the access token is invalid.
            // In this case, we'll get an exception, so we'll
            // just ask the user to login again here.
            $login_url = $facebook->getLoginUrl();
            $facebook_html .= l($this->t('Please you need to connect to the Facebook account which will display selected contents'), $login_url);
            error_log($e->getType());
            error_log($e->getMessage());
          }   
        }
        else {
          // No user, print a link for the user to login.
          $login_url = $facebook->getLoginUrl();
          $facebook_html .= l($this->t('Please you need to connect to the Facebook account which will display selected contents'), $login_url);
        }
      }
    }
    else {
      $fb_profile = $publish_away_config->get('facebook.user_profile');
      $facebook_html .= '<strong>' . $this->t('Facebook account') . ' :</strong> ' . $publish_away_config->get('facebook.name');
      if (isset($fb_profile['quotes']) && trim($fb_profile['quotes']) != '') {
        $facebook_html .= '<br /><strong>' . $this->t('Last Facebook status') . ' :</strong> ' . $fb_profile['quotes'];
      }
      $logout_url = 'admin/config/services/publish-away/fb-logout';
      $facebook_html .= ' <a href="' . url($logout_url) . '">' . $this->t('Logout') . '</a><br />';
    }
    return $facebook_html;
  }

  private function twitter_login() {
    $publish_away_config = $this->configFactory->get('publish_away.settings');
    $twitter_html = '';
    $consumer_key = $publish_away_config->get('twitter.consumer_key');
    $consumer_secret = $publish_away_config->get('twitter.consumer_secret');
    $oauth_token = $publish_away_config->get('twitter.oauth_token');
    $oauth_token_secret = $publish_away_config->get('twitter.oauth_token_secret');

    if ($consumer_key != '' && $consumer_secret != '' && $oauth_token != '' && $oauth_token_secret != '') {
      module_load_include('inc', 'publish_away', 'twitteroauth');
      $publish_away_twitter = new TwitterOAuth($consumer_key, $consumer_secret, $oauth_token, $oauth_token_secret);
      if (!empty($publish_away_twitter)) {
        $twitter_user = $publish_away_twitter->get('account/verify_credentials');
        if (isset($twitter_user->id)) {
          $twitter_html .= '<strong>' . $this->t('Twitter account') . ' :</strong> ' . $twitter_user->name;
          if (isset($twitter_user->status) && isset($twitter_user->status->text) && trim($twitter_user->status->text) != '') {
            $twitter_html .= '<br /><strong>' . $this->t('Last Tweet') . ' :</strong> ' . $twitter_user->status->text;
          }
          $twitter_html .= '<br /><img style="veryical-align:middle;" src="'. $twitter_user->profile_image_url .'" alt="'. $this->t('User image') .'" />';
        }
        else {
          $this->configFactory->get('publish_away.settings')
            ->set('twitter.consumer_key', '')
            ->save();
        }
      }
    }
    return $twitter_html;
  }

  private function admin_facebook_logout() {
    // Log out the Facebook acount.
    $config = array(
      'appId'  => $publish_away_config->get('facebook.appId'),
      'secret' => $publish_away_config->get('facebook.secret'),   
    );
    $facebook = new Facebook($config);
    publish_away_del('publish_away_fb_token');
    publish_away_del('publish_away_fb_id');
    publish_away_del('publish_away_fb_name');
    $this->configFactory->get('publish_away.settings')
      ->set('facebook.token', '')
      ->set('facebook.id', '')
      ->set('facebook.name', '')
      ->set('facebook.user_profile', '')
      ->save();
    $facebook->api('/' . publish_away_get('publish_away_fb_id', 'me') . '/permissions', 'DELETE');
    // drupal_goto(url('admin/config/services/publish-away'));
  }
}
